#ifndef BLD_INTERP_H
#define BLD_INTERP_H

/******************************************************************************

Copyright (c) 1997, Andrew J. Weber, Jason P. Weber

Copyright information should have been included with this distribution
and may be found at ../meta/COPYRIGHT or obtained via email from weber@acm.org

******************************************************************************/

#include <gpl/gpl.h>
#include <wba/wba.h>
#include <bld/context.h>
#include <bld/rule.h>
#include <bld/state.h>

#ifdef	BLD_APP
#undef	BLD_APP
#endif	/* BLD_APP */
#define	BLD_APP	((BLD_Interpreter *) WBA_Application::GetCurrentApplication())
#ifdef	BLD_CONTEXT
#undef	BLD_CONTEXT
#endif	/* BLD_CONTEXT */
#define	BLD_CONTEXT	(*(((BLD_Interpreter *) WBA_Application::GetCurrentApplication())->GetCurrentContext()))
#ifdef	BLD_STATE
#undef	BLD_STATE
#endif	/* BLD_STATE */
#define	BLD_STATE(x)	(((BLD_Interpreter *) WBA_Application::GetCurrentApplication())->FindState(x))
#ifdef	BLD_ARG_ON
#undef	BLD_ARG_ON
#endif	/* BLD_ARG_ON */
#define	BLD_ARG_ON(x)	((WBA_Application::GetCurrentApplication()->GetEnvironment()->GetArgumentTable())->Find(x)->IsTrue())
#ifdef	BLD_ARG_OFF
#undef	BLD_ARG_OFF
#endif	/* BLD_ARG_OFF */
#define	BLD_ARG_OFF(x)	((WBA_Application::GetCurrentApplication()->GetEnvironment()->GetArgumentTable())->Find(x)->IsFalse())

/***********************************************************************
class BLD_Interpreter

***********************************************************************/
class BLD_Interpreter : public WBA_Application
{
	public:
		// Public place-holders for yyparse().
		// Of course, this is rather inconvenient for multi-threading.
		static char *parseFileName;
		static GPL_Boolean parseSuccess;
		static unsigned long lineNumber;
		static OpArray *operations;

		BLD_Interpreter(char **e = (char **) 0);
		virtual ~BLD_Interpreter(void);

		BLD_Context *GetCurrentContext(void) { return contexts.Top(); }
		virtual const char *GetName(void) const{ return "Builder"; }
		virtual const char *GetParameterUsage(void) const { return "[target...]"; }

		virtual GPL_Boolean Initialize(void);
		virtual void Finalize(void);
		virtual GPL_Boolean Parse(const GPL_String & fileName, OpArray *ops);
		virtual GPL_Boolean Build(const GPL_String & fileName,
									const GPL_Array<GPL_String> & names);
		void AddRule(const GPL_String & string, const BLD_RuleType type,
						const OpArray & ops);
		BLD_State & FindState(const GPL_String & name);
		GPL_Boolean GetModificationTime(const GPL_String & fileName,
										GPL_Time & time);

		void DumpStates(void);
		void DumpEnvironment(void);

	protected:
		GPL_Stack<BLD_Context *> contexts;
		GPL_Map<GPL_String, BLD_State> states;
		char **envp;
		static const GPL_String defaultTargetName;

		virtual void Process(void);
};

#endif /* BLD_INTERP_H */
