
/******************************************************************************

Copyright (c) 1997, Andrew J. Weber, Jason P. Weber

Copyright information should have been included with this distribution
and may be found at ../meta/COPYRIGHT or obtained via email from weber@acm.org

******************************************************************************/

#include <osd/osd.h>
#include <gpl/gpl.h>
#include <bld/fnop.h>
#include <bld/scope.h>
#include <bld/block.h>
#include <bld/target.h>
#include <bld/interp.h>

/***********************************************************************
class BLD_Target
***********************************************************************/
GPL_Stack<BLD_Target*> BLD_Target::targets;

BLD_Target::BLD_Target(void)
{
}

BLD_Target::~BLD_Target(void)
{
}

BLD_BuildStatus BLD_Target::Build(const GPL_Boolean reuseScope)
{
	// If this is a nameless target...
	if (0 == name.GetLength())
	{
		return BLD_BUILD_UNNECESSARY;
	}

	// Push oneself onto the stack.
	targets.Push(this);

	// Perform the operations, relying on each operation
	// to set the EXECUTED_COMMANDS and BUILD_FAILED
	// bits where appropriate.
	block.Execute(reuseScope);

	// Pop oneself off the stack.
	targets.Pop();

	// Return a value based on the success of the operations.
	if (BLD_STATE(name).GetFailed())
	{
		WBA_APP->Printf("Target %s failed", name.GetData());
		return BLD_BUILD_FAILED;
	}

	if (BLD_STATE(name).GetExecuted())
	{
		// Update the build time.
		BLD_STATE(name).SetLastBuilt();

		return BLD_BUILD_NEEDED;
	}
	else
	{
		if (!BLD_STATE(name).HasTimeStamp())
		{
			BLD_STATE(name).SetLastBuilt();
		}
		return BLD_BUILD_UNNECESSARY;
	}
}

GPL_Boolean BLD_Target::AddRule(const BLD_RuleBase * rule,
								const GPL_Array<GPL_String> & expressions)
{
	if (BLD_RULE_OVERWRITE_OPERATIONS == rule->GetType () &&
		block.HasPrimarySubBlock())
	{
		if (BLD_ARG_ON("BLD_DEBUG"))
		{
			WBA_APP->Printf("Over-writing primary operations for target %s",
							name.GetData());
		}
	}
	block.AddRule(rule, expressions);
	return TRUE;
}

void BLD_Target::SetExecuted(void)
{
	if (0 < BLD_Target::targets.GetCount())
	{
		if (0 < BLD_Target::targets.Top()->name.GetLength())
		{
			// Mark this target has having executed.
			const GPL_String fullPathName =
				GPL_File::GetFullPathName(BLD_Target::targets.Top()->name);
			BLD_STATE(fullPathName).SetExecuted();
		}
	}
}

void BLD_Target::SetFailed(void)
{
	if (0 < BLD_Target::targets.GetCount())
	{
		if (0 < BLD_Target::targets.Top()->name.GetLength())
		{
			// Mark this target has having failed.
			const GPL_String fullPathName =
				GPL_File::GetFullPathName(BLD_Target::targets.Top()->name);
			BLD_STATE(fullPathName).SetFailed();
		}

		// Terminate execution for this target.
		BLD_Target::targets.Top()->Terminate();
	}
}

